/* ------------------------------------------------------------------------------------
 * Un exemple d'implmentation du cache d'un fichier afin d'explorer
 * l'effet des algorithmes de gestion et de remplacement
 * ------------------------------------------------------------------------------------
 * Jean-Paul Rigault
 * $Id$
 * ------------------------------------------------------------------------------------
 * Interface avec une bibliothque de stratgie charge dynamiquement
 * ------------------------------------------------------------------------------------
 */

#ifndef _LOAD_STRATEGY_H_
#define _LOAD_STRATEGY_H_

/* ------------------------------------------------------------------------------------
 *  Dfinition du type numr dsignant les fonctions d'interface
 * ------------------------------------------------------------------------------------
*/

enum Strategy_Function
{
    STRATEGY_CREATE,		/* == 0 */
    STRATEGY_CLOSE,
    STRATEGY_INVALIDATE,
    STRATEGY_REPLACE,
    STRATEGY_READ,
    STRATEGY_WRITE,
    STRATEGY_NAME
};

static const int N_STRATEGY_FUNCTIONS = STRATEGY_NAME + 1;

/* ------------------------------------------------------------------------------------
 *  Dfinition du tableau de correspondance entre le numro de la fonction et un 
 * pointeur sur son code
 * ------------------------------------------------------------------------------------
*/

/* La structure de correspondance elle-mme */

typedef void (*ANY_PTR_FUNC)();

struct Strategy_Interface
{
    char *fname;	/* le nom de la fonction */
    ANY_PTR_FUNC fptr;	/* un pointeur sur son code : on prend un pointeur sur
                         * fonction --- de type queconque --- pour viter un
                         * warning */
};

/* Le tableau de correspondance (statique, car dfini dans un .h */
struct Strategy_Interface strategy_functions[] = 
{
    {"Strategy_Create", NULL},		/* indice STRATEGY_CREATE */
    {"Strategy_Close", NULL},
    {"Strategy_Invalidate", NULL},
    {"Strategy_Replace_Block", NULL},
    {"Strategy_Read", NULL},
    {"Strategy_Write", NULL},
    {"Strategy_Name", NULL},
};

/* ------------------------------------------------------------------------------------
 *  Dfinition des types de fonctions d'interface 
 * ------------------------------------------------------------------------------------
*/

/* Ceci est le type  Strategy_Create()
 * 	struct Cache * --> void *
 */
typedef void *(*STRATEGY_CREATE_TYPE)(struct Cache *);

/* Ceci est le type  Strategy_Close() et Strategy_Invalidate()
 * 	struct Cache * --> void
 */
typedef void (*STRATEGY_CLOSE_INVALIDATE_TYPE)(struct Cache *);

/* Ceci est le type de Strategy_Replace_Block()
 * 	struct Cache * --> struct Cache_Block_Header *
 */
typedef struct Cache_Block_Header *(*STRATEGY_REPLACE_TYPE)(struct Cache *);

/* Ceci est le type de Strategy_Read() et Strategy_Write()
 * 	struct Cache * x struct Cache_Block_Header * --> void
 */
typedef void (*STRATEGY_READ_WRITE_TYPE)(struct Cache *, struct Cache_Block_Header *);

/* Ceci est le type de Strategy_Name
 * 	void --> char *
 */
typedef char *(*STRATEGY_NAME_TYPE)(void);

/* ------------------------------------------------------------------------------------
 * Macro de transtypage des fonctions de la stratgie
 * Convertit le pointeur sur fonction d'indice strategy_func (un enum Strategy_Func)
 * en type
 * ------------------------------------------------------------------------------------
*/

#define STRATEGY_FUNC(type, sfunc) ((type)(strategy_functions[sfunc].fptr))

/* ------------------------------------------------------------------------------------
 * Nom de la bibliothque de stratgie (juste le prfixe)
 * ------------------------------------------------------------------------------------
*/

/* Prfixe du nom */
static const char *libname_prefix = "libStrategy-" ; 
static const char *libname_suffix = ".so" ; 

/* Longueur totale du nom : ceci devrait suffire */
#define NLIBNAME 50

/* ------------------------------------------------------------------------------------
 * Nom de la stratgie
 * ------------------------------------------------------------------------------------
*/

extern char *The_Strategy;

#endif /* _LOAD_STRATEGY_H_ */
